<?php
/**
 * Plugin Name: Bankful
 * Plugin URI: https://woo.com/products/bankful/
 * Description: Accept credit card payments, bankful hosted payments page and supports WooCommerce subscriptions on your store.
 * Version: 2.0.6
 * Author: Bankful
 * Author URI: https://bankful.com/
 * Developer: Bankful
 * Developer URI: https://bankful.com/
 * Text Domain: bankful
 *
 * Woo: 18734002743901:a5038511e438bfee33cba3afbcc86f03
 * WC requires at least: 7.0
 * WC tested up to: 8.5.2
 *
 * License: GNU General Public License v3.0
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 */

if (!defined('ABSPATH')) { // To check absolute path
	exit;
}

function verifyHashSignatureFromRequest( $params ) {
	$bankful_options = get_option( 'bankful_options' );
	$testEnv = ( strtolower($bankful_options['bankful_field_environment'])==strtolower('Test') );
	$req_password   = $testEnv ? ( isset( $bankful_options[ 'bankful_field_test_password' ] ) ? $bankful_options[ 'bankful_field_test_password' ] : '' ) : ( isset( $bankful_options[ 'bankful_field_live_password' ] ) ? $bankful_options[ 'bankful_field_live_password' ] : '' );
	if (!$req_password) {
		return false;
	}
	$signatureGenerated = generateSignatureHash($params, $req_password);

	// write_logs_bf('SIGNATURE--------------------------');
	// write_logs_bf($signatureGenerated);
	// write_logs_bf($params['SIGNATURE']);
	// write_logs_bf('SIGNATURE--------------------------');
	return $signatureGenerated === $params['SIGNATURE'];
}

function generateSignatureHash( $data, $salt ) {
	$notInclude = array( 'SIGNATURE' );

	$strField = '';

	$paramData = array_filter(array_keys($data), function ( $key ) use ( $notInclude ) {
		return !in_array($key, $notInclude);
	});

	sort($paramData);
	// write_logs_bf(print_r($paramData, true));
	foreach ($paramData as $paramName) {
		if ('' !== $data[$paramName]) {
			$strField .= $paramName . $data[$paramName];
		}
	}
	// MAKE HASH SHA 256
	// write_logs_bf(print_r($strField, true));

	$generatedSignature = hash_hmac('sha256', $strField, $salt);

	return $generatedSignature;
}

// Load all Classes and actions / filters.
add_action('plugins_loaded', 'Bf_Init_Gateway_classes');

// Global Variables
$WP_SITE_URL = get_site_url();
$BF_PLUGIN_SOURCE = 'woo_market';

/**
 * The class itself, please note that it is inside plugins_loaded action hook
 *
 */
function Bf_Init_Gateway_classes() {

	// Check class WP_Payment_Gateway exist or not
	if (!class_exists('WC_Payment_Gateway')) {
		return;
	}

	if ( class_exists( 'WC_Bankful_WS_Gateway' ) || class_exists('WC_Gateway_Bankful') ) {
		add_action( 'admin_notices', 'bankful_gateway_exists_error' );
		return;
	}

	// Credit Card Gateway with Woo Subscriptions
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-wc-bankful-ws-gateway.php';

	// Hosted Page Bankful Gateway
	require_once plugin_dir_path(__FILE__) . '/includes/bankful-webhook-controller.php';
	require_once plugin_dir_path(__FILE__) . '/includes/class-wc-bankful-hosted-gateway.php';
	require_once plugin_dir_path(__FILE__) . '/includes/blocks/class-wc-bankful-payments-blocks.php';
	require_once plugin_dir_path(__FILE__) . '/includes/blocks/class-wc-bankful-ws-payments-blocks.php';
	// Actions and Filters
	add_action('admin_menu', 'add_bankful_menus', 60);
	add_action('admin_enqueue_scripts', 'enqueue_styles_scripts_bankful');
	add_action('wp_enqueue_scripts', 'front_enqueue_styles_bankful');
	// Registers WooCommerce Blocks integration.
	add_filter('woocommerce_payment_gateways', 'Bf_Add_Gateway_classes');
	
	add_action('woocommerce_blocks_loaded', 'woocommerce_gateway_bankful_woocommerce_block_support');
	add_filter('http_request_timeout', 'wp_timeout_extend_fb_one');
	add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'plugin_action_links_bankful');
}

/**
 * Plugin url.
 *
 * @return string
 */
function plugin_url() {
	return untrailingslashit( plugins_url( '/', __FILE__ ) );
}

/**
 * Plugin url.
 *
 * @return string
 */
function plugin_abspath() {
	return trailingslashit( plugin_dir_path( __FILE__ ) );
}

/**
 * Registers WooCommerce Blocks integration.
 *
 */
function woocommerce_gateway_bankful_woocommerce_block_support() {
	$options = get_option('bankful_options');

	if (isset($options['bankful_field_environment']) && 'Test' == $options['bankful_field_environment']) {
		if (isset($options['bankful_field_test_validated_account_cc']) && 'yes' == $options['bankful_field_test_validated_account_cc']) {
			if (class_exists('Automattic\WooCommerce\Blocks\Payments\Integrations\AbstractPaymentMethodType')) {
				add_action(
					'woocommerce_blocks_payment_method_type_registration',
					function ( Automattic\WooCommerce\Blocks\Payments\PaymentMethodRegistry $payment_method_registry ) {
						$payment_method_registry->register(new WC_Bankful_WS_Gateway_Blocks_Support());
					}
				);
			}
		}
		if (isset($options['bankful_field_test_validated_account_hg']) && 'yes' == $options['bankful_field_test_validated_account_hg']) {
			if (class_exists('Automattic\WooCommerce\Blocks\Payments\Integrations\AbstractPaymentMethodType')) {
				add_action(
					'woocommerce_blocks_payment_method_type_registration',
					function ( Automattic\WooCommerce\Blocks\Payments\PaymentMethodRegistry $payment_method_registry ) {
						$payment_method_registry->register(new WC_Gateway_Bankful_Blocks_Support());
					}
				);
			}
		}
	} else if (isset($options['bankful_field_environment']) && 'Live' == $options['bankful_field_environment']) {

		if (isset($options['bankful_field_live_validated_account_cc']) && 'yes' == $options['bankful_field_live_validated_account_cc']) {
			if (class_exists('Automattic\WooCommerce\Blocks\Payments\Integrations\AbstractPaymentMethodType')) {
				add_action(
					'woocommerce_blocks_payment_method_type_registration',
					function ( Automattic\WooCommerce\Blocks\Payments\PaymentMethodRegistry $payment_method_registry ) {
						$payment_method_registry->register(new WC_Bankful_WS_Gateway_Blocks_Support());
					}
				);
			}
		}
		if (isset($options['bankful_field_live_validated_account_hg']) && 'yes' == $options['bankful_field_live_validated_account_hg']) {
			if (class_exists('Automattic\WooCommerce\Blocks\Payments\Integrations\AbstractPaymentMethodType')) {
				add_action(
					'woocommerce_blocks_payment_method_type_registration',
					function ( Automattic\WooCommerce\Blocks\Payments\PaymentMethodRegistry $payment_method_registry ) {
						$payment_method_registry->register(new WC_Gateway_Bankful_Blocks_Support());
					}
				);
			}
		}
	}
}

/**
 * Enqueue custom CSS and scripts for Bankful Gateway Frontend
 *
 */
function front_enqueue_styles_bankful() {
	wp_register_style('bankful_front_screens', plugin_dir_url(__FILE__) . 'assets/css/bf-style.css', false, '2.0.6');
	wp_enqueue_style('bankful_front_screens');
}


/**
 * Add links to plugins page for checkout settings
 *
 * @param  array $links
 * @return array
 */
function plugin_action_links_bankful( $links = array() ) {

	$options = get_option( 'bankful_options' );
	$plugin_links = array( '<a href="' . admin_url('admin.php?page=bankful_settings') . '">' . __('Bankful', 'bankful') . '</a>' );

	if (isset($options['bankful_field_environment']) && 'Test' == $options['bankful_field_environment']) {

		( isset($options['bankful_field_test_validated_account_cc']) && 'yes' == $options['bankful_field_test_validated_account_cc'] ) ?
		$plugin_links[] = '<a href="' . admin_url('admin.php?page=wc-settings&tab=checkout&section=bankful_credit_card') . '">' . __('Credit Card', 'bankful') . '</a>' : '';
		( isset($options['bankful_field_test_validated_account_hg']) && 'yes' == $options['bankful_field_test_validated_account_hg'] ) ?
		$plugin_links[] = '<a href="' . admin_url('admin.php?page=wc-settings&tab=checkout&section=bankful_hosted_gateway') . '">' . __('Bankful Hosted', 'bankful') . '</a>' : '';

	} else if (isset($options['bankful_field_environment']) && 'Live' == $options['bankful_field_environment']) {

		( isset($options['bankful_field_live_validated_account_cc']) && 'yes' == $options['bankful_field_live_validated_account_cc'] ) ?
		$plugin_links[] = '<a href="' . admin_url('admin.php?page=wc-settings&tab=checkout&section=bankful_credit_card') . '">' . __('Credit Card', 'bankful') . '</a>' : '';
		( isset($options['bankful_field_live_validated_account_hg']) && 'yes' == $options['bankful_field_live_validated_account_hg'] ) ?
		$plugin_links[] = '<a href="' . admin_url('admin.php?page=wc-settings&tab=checkout&section=bankful_hosted_gateway') . '">' . __('Bankful Hosted', 'bankful') . '</a>' : '';

	}

	return array_merge($plugin_links, $links);
}

/**
 * This function is used to log on globally for bankful plugin
 *
 * @param string $message message to log
 *
 * @param string $level log level like info/debug
 */
function write_logs_bf( $message = '', $level = 'info' ) {
	$logger = wc_get_logger();

	$logger->log($level, wc_print_r( $message, true ), array( 'source' => 'bankful_payment_gateway' ));
}

/**
 * This action hook registers notice that bankful gateway plugin already active
 *
 * @param string $gateways SDK/API method name
 *
 * @return string error message
 */
function bankful_gateway_exists_error() {
	$return_error = '<div class="notice error my-acf-notice is-dismissible" >';
	$return_error .= '<p>' . esc_html_e( 'Another Bankful Gateway already activated, please deactivate other Bankful or Bankful Hosted Gateway. Thanks.', 'bankful' ) . '</p>';
	$return_error .= '</div>';
	return $return_error;
}

/**
 * This action hook registers our PHP class as a WooCommerce payment gateway
 *
 * @param string $gateways SDK/API method name
 *
 * @return object  $this
 */
function Bf_Add_Gateway_classes( $gateways ) {
	$options = get_option( 'bankful_options' );

	if (isset($options['bankful_field_environment']) && 'Test' == $options['bankful_field_environment']) {

		( isset($options['bankful_field_test_validated_account_cc']) && 'yes' == $options['bankful_field_test_validated_account_cc'] ) ?
		$gateways[] = 'WC_Bankful_WS_Gateway' : '';
		( isset($options['bankful_field_test_validated_account_hg']) && 'yes' == $options['bankful_field_test_validated_account_hg'] ) ?
		$gateways[] = 'WC_Gateway_Bankful' : '';

	} else if (isset($options['bankful_field_environment']) && 'Live' == $options['bankful_field_environment']) {

		( isset($options['bankful_field_live_validated_account_cc']) && 'yes' == $options['bankful_field_live_validated_account_cc'] ) ?
		$gateways[] = 'WC_Bankful_WS_Gateway' : '';
		( isset($options['bankful_field_live_validated_account_hg']) && 'yes' == $options['bankful_field_live_validated_account_hg'] ) ?
		$gateways[] = 'WC_Gateway_Bankful' : '';

	}

	return $gateways;
}

/**
 * Process the payment and return the result.
 *
 * @param datetime $time Order ID.
 *
 * @return array
 */
function wp_timeout_extend_fb_one( $time ) {
	// Default timeout is 5
	return 30;
}

/**
 * Add Bankful Menus to Admin Menus
 *
 */
function add_bankful_menus() {
	add_submenu_page(
		'woocommerce',
		'Bankful',
		'Bankful',
		'manage_woocommerce',
		'bankful_settings',
		'bankful_options_page_html'
	);
}


/**
 * Top level menu callback function for bankful_options_page_html
 */
function bankful_options_page_html() {
	// check user capabilities
	if ( ! current_user_can( 'manage_options' ) ) {
		return;
	}

	// check if the user have submitted the settings
	// WordPress will add the "settings-updated" $_GET parameter to the url
	if ( isset( $_GET['settings-updated'] ) ) {
		// add settings saved message with the class of "updated"
		add_settings_error( 'bankful_messages', 'bankful_message', __( 'Settings Saved', 'bankful' ), 'updated' );
	}

	$options = get_option( 'bankful_options' );

	// show error/update messages
	settings_errors( 'bankful_messages' );
	?>
	<div class="wrap">
		<h1><?php echo esc_html( get_admin_page_title() ); ?></h1>
		<form action="options.php" method="post" name="bankful_settings">
			<?php
			// output security fields for the registered setting "bankful"
			settings_fields( 'bankful' );
			// output setting sections and their fields
			// (sections are registered for "bankful", each field is registered to a specific section)
			do_settings_sections( 'bankful' );
			?>
			<input type="button" name="submitValidate" id="submitValidate" class="button" value="Sync Settings">
			<p>
			<?php
			if (isset($options['bankful_field_environment']) && 'Test' == $options['bankful_field_environment']) {

				echo( ( isset($options['bankful_field_test_validated_account_cc']) && 'yes' == $options['bankful_field_test_validated_account_cc'] ) ?
				'<a style="margin-right: 10px;" type="button" name="redirectToCC" id="redirectToCC" class="button" href="../wp-admin/admin.php?page=wc-settings&tab=checkout&section=bankful_credit_card">Credit Card Settings</a>' : '' );
				echo ( ( isset($options['bankful_field_test_validated_account_hg']) && 'yes' == $options['bankful_field_test_validated_account_hg'] ) ?
				'<a type="button" name="redirectToHP" id="redirectToHP" class="button" href="../wp-admin/admin.php?page=wc-settings&tab=checkout&section=bankful_hosted_gateway">Hosted Page Settings</a>' : '' );

			} else if (isset($options['bankful_field_environment']) && 'Live' == $options['bankful_field_environment']) {

				echo( ( isset($options['bankful_field_live_validated_account_cc']) && 'yes' == $options['bankful_field_live_validated_account_cc'] ) ?
				'<a style="margin-right: 10px;" type="button" name="redirectToCC" id="redirectToCC" class="button" href="">Credit Card Settings</a>' : '' );
				echo( ( isset($options['bankful_field_live_validated_account_hg']) && 'yes' == $options['bankful_field_live_validated_account_hg'] ) ?
				'<a type="button" name="redirectToHP" id="redirectToHP" class="button" href="">Hosted Page Settings</a>' : '' );
			}
			// output save settings button
			submit_button( 'Save Settings', 'hidden' );
			?>
			</p>
		</form>
	</div>
	<script>
		jQuery('#submitValidate').on('click', async function(event) {
			const test_username = jQuery("#bankful_field_test_username").val();
			const test_password = jQuery("#bankful_field_test_password").val();

			const live_username = jQuery("#bankful_field_live_username").val();
			const live_password = jQuery("#bankful_field_live_password").val();

			const validated_test_username = `${test_username.substr(0, 2)}*****${test_username.substr(test_username.length - 8)}`;

			const validated_live_username = `${live_username.substr(0, 2)}*****${live_username.substr(live_username.length - 8)}`;

			const environment = jQuery("#bankful_field_environment").val();

			const current_site_url = '<?php echo esc_url(get_site_url()); ?>';

			// const request_url_test = "http://localhost:50001/";
			const request_url_live = "https://api.paybybankful.com/";
			const request_url_test = 'https://api-qa.bankfulportal.com/';

			console.log(environment);
			console.log(current_site_url);

			let live_saved = false;
			let test_saved = false;
			if(live_username && live_password) {
				await jQuery.ajax({
					type : "post",
					dataType : "json",
					url : request_url_live + 'api/woocommerce/validate-credentials',
					data : {req_username: live_username, req_password: live_password, site_url: current_site_url},
					success: function(response) {

						if('success' === response.status) {
							const data = response.data;
							jQuery("#bankful_field_live_validate").val(validated_live_username);
							jQuery("#bankful_field_live_validated_account_hg").val(data.isHostedPageEnabled === "Y" ? 'yes' : "no");
							jQuery("#bankful_field_live_validated_account_cc").val(data.isNativeEnabled === "Y" ? 'yes' : "no");
							jQuery("#bankful_field_live_subscription_status").val(data.isSubscriptionEnabled === "Y" ? 'yes' : "no");
							jQuery(".description.bankful_field_live_password").html('<span style="color:green;">Validation Success.</span>');

							if(environment === "Live") {
								jQuery('#submit').trigger("click");
							}
						} else {
							jQuery(".description.bankful_field_live_password").html('<span style="color:red;">Validation Failed.</span>');
							jQuery("#bankful_field_live_validate").val("");
							jQuery("#bankful_field_live_validated_account_hg").val("no");
							jQuery("#bankful_field_live_validated_account_cc").val("no");
							jQuery("#bankful_field_live_subscription_status").val("no");
							alert(response.errorMessage);
							jQuery('#submit').trigger("click");
						}
					}
				});

			} else {
				jQuery(".description.bankful_field_live_password").html('<span style="color:red;">Please enter Live Username and Password.</span>');
			}

			if(test_username && test_password) {
				await jQuery.ajax({
					type : "post",
					dataType : "json",
					url : request_url_test + 'api/woocommerce/validate-credentials',
					data : {req_username: test_username, req_password: test_password, site_url: current_site_url},
					success: function(response) {
						// if(response.type == "success") {

						if('success' === response.status) {
							const data = response.data;
							jQuery("#bankful_field_test_validate").val(validated_test_username);
							jQuery("#bankful_field_test_validated_account_hg").val(data.isHostedPageEnabled === "Y" ? 'yes' : "no");
							jQuery("#bankful_field_test_validated_account_cc").val(data.isNativeEnabled === "Y" ? 'yes' : "no");
							jQuery("#bankful_field_test_subscription_status").val(data.isSubscriptionEnabled === "Y" ? 'yes' : "no");
							jQuery(".description.bankful_field_test_password").html('<span style="color:green;">Validation Success.</span>');

							if(environment === "Test") {
								jQuery('#submit').trigger("click");
							}
						} else {
							jQuery(".description.bankful_field_test_password").html('<span style="color:red;">Validation Failed.</span>');
							jQuery("#bankful_field_test_validate").val("");
							jQuery("#bankful_field_test_validated_account_hg").val("no");
							jQuery("#bankful_field_test_validated_account_cc").val("no");
							jQuery("#bankful_field_test_subscription_status").val("no");
							alert(response.errorMessage);
							jQuery('#submit').trigger("click");
						}
					}
				});

			} else {
				jQuery(".description.bankful_field_live_password").html('<span style="color:red;">Please enter Test Username and Password.</span>');
			}

		});

		jQuery('#TEST_edit').on('click', function(event) {
			event.preventDefault();
			jQuery("#bankful_field_test_validate").val("");
			jQuery("#bankful_field_test_validated_account_hg").val("no");
			jQuery("#bankful_field_test_validated_account_cc").val("no");
			jQuery("#bankful_field_test_subscription_status").val("no");
			jQuery('#submit').trigger("click");
		});

		jQuery('#LIVE_edit').on('click', function(event) {
			event.preventDefault();
			jQuery("#bankful_field_live_validate").val("");
			jQuery("#bankful_field_live_validated_account_hg").val("no");
			jQuery("#bankful_field_live_validated_account_cc").val("no");
			jQuery("#bankful_field_live_subscription_status").val("no");
			jQuery('#submit').trigger("click");
		});
	</script>
	<?php
}

/**
 * Redirect to WooCommerce Payment Gateway Settings Page
 *
 */
function redirect_to_gateway_bankful_cc() {
	wp_safe_redirect(admin_url('admin.php?page=wc-settings&tab=checkout&section=bankful_credit_card'));
	exit;
}

/**
 * Redirect to WooCommerce Payment Gateway Settings Page
 *
 */
function redirect_to_gateway_bankful_hg() {
	wp_safe_redirect(admin_url('admin.php?page=wc-settings&tab=checkout&section=bankful_hosted_gateway'));
	exit;
}

/**
 * Enqueue custom CSS and scripts for Bankful Gateeway admin
 *
 */
function enqueue_styles_scripts_bankful() {
	wp_register_style('bankful_admin_screens', plugin_dir_url(__FILE__) . 'assets/css/admin-bf.css', false, '2.0.6');
	wp_enqueue_style('bankful_admin_screens');
}

/**
 * Register our bankful_settings_init to the admin_init action hook.
 */
add_action( 'admin_init', 'bankful_settings_init' );

/**
 * Bankful option and settings
 */
function bankful_settings_init() {
	// Register a new setting for "bankful" page.
	register_setting( 'bankful', 'bankful_options' );

	$options = get_option( 'bankful_options' );
	$hidden_validated = ( isset($options['bankful_field_test_validate']) && !empty($options['bankful_field_test_validate']) ) ? 'hidden' : '';
	$hidden_not_validated = ( !isset($options['bankful_field_test_validate']) || empty($options['bankful_field_test_validate']) ) ? 'hidden' : '';

	$hidden_validated_live = ( isset($options['bankful_field_live_validate']) && !empty($options['bankful_field_live_validate']) ) ? 'hidden' : '';
	$hidden_not_validated_live = ( !isset($options['bankful_field_live_validate']) || empty($options['bankful_field_live_validate']) ) ? 'hidden' : '';
	
	$hidden_subscription = ( ( isset($options['bankful_field_test_subscription_status']) && 'yes' === $options['bankful_field_test_subscription_status'] ) || 
	( isset($options['bankful_field_live_subscription_status']) && 'yes' === $options['bankful_field_live_subscription_status'] ) ) ? '' : 'hidden';

	// Register a new section in the "bankful" page.
	add_settings_section(
		'bankful_section_settings',
		__( 'API Credentials', 'bankful' ),
		'bankful_section_settings_callback',
		'bankful'
	);

	// Register a new field in the "bankful_field_test_cb" section, inside the "bankful" page.
	add_settings_field(
		'bankful_field_environment',
		__( 'Current Environment:', 'bankful' ),
		'bankful_field_dropdown',
		'bankful',
		'bankful_section_settings',
		array(
			'label_for'         => 'bankful_field_environment',
			'class'             => 'bankful_row',
			'type'              => 'text',
			'desc'              => '',
			'default'           => 'Live',
		)
	);

	// Register a new field in the "bankful_field_test_cb" section, inside the "bankful" page.
	add_settings_field(
		'bankful_field_test_validate',
		__( 'Test API Status:', 'bankful' ),
		'bankful_field_test_cb',
		'bankful',
		'bankful_section_settings',
		array(
			'label_for'         => 'bankful_field_test_validate',
			'class'             => 'bankful_row disabled ' . $hidden_not_validated,
			'type'              => 'text',
			'desc'              => '',
			'default'           => '',
			'info'              => 'TEST',
		)
	);

	// Register a new field in the "bankful_field_test_cb" section, inside the "bankful" page.
	add_settings_field(
		'bankful_field_test_validated_account_cc',
		__( 'Test API Connected Account Credit Card:', 'bankful' ),
		'bankful_field_test_cb',
		'bankful',
		'bankful_section_settings',
		array(
			'label_for'         => 'bankful_field_test_validated_account_cc',
			'class'             => 'bankful_row hidden ',
			'type'              => 'text',
			'desc'              => '',
			'default'           => 'no',
		)
	);

	// Register a new field in the "bankful_field_test_cb" section, inside the "bankful" page.
	add_settings_field(
		'bankful_field_test_validated_account_hg',
		__( 'Test API Connected Account Hosted Gateway:', 'bankful' ),
		'bankful_field_test_cb',
		'bankful',
		'bankful_section_settings',
		array(
			'label_for'         => 'bankful_field_test_validated_account_hg',
			'class'             => 'bankful_row hidden ',
			'type'              => 'text',
			'desc'              => '',
			'default'           => 'no',
		)
	);

	// Register a new field in the "bankful_field_test_cb" section, inside the "bankful" page.
	add_settings_field(
		'bankful_field_test_username',
		__( 'Test API Username', 'bankful' ),
		'bankful_field_test_cb',
		'bankful',
		'bankful_section_settings',
		array(
			'label_for'         => 'bankful_field_test_username',
			'class'             => 'bankful_row ' . $hidden_validated,
			'type' => 'text',
			'desc' => 'Add Test Username here.',
			'default' => 'sandbox-test@bankful.com',
		)
	);

	// Register a new field in the "bankful_field_test_cb" section, inside the "bankful" page.
	add_settings_field(
		'bankful_field_test_password',
		__( 'Test API Password', 'bankful' ),
		'bankful_field_test_cb',
		'bankful',
		'bankful_section_settings',
		array(
			'label_for'         => 'bankful_field_test_password',
			'class'             => 'bankful_row ' . $hidden_validated,
			'type' => 'password',
			'desc' => 'Add Test Password here.',
			'default' => 'tuMcC7m333J4VMsb',
		)
	);

	// Register a new field in the "bankful_field_test_cb" section, inside the "bankful" page.
	add_settings_field(
		'bankful_field_live_validate',
		__( 'Live API Status:', 'bankful' ),
		'bankful_field_test_cb',
		'bankful',
		'bankful_section_settings',
		array(
			'label_for'         => 'bankful_field_live_validate',
			'class'             => 'bankful_row disabled ' . $hidden_not_validated_live,
			'type'              => 'text',
			'desc'              => '',
			'default'           => '',
			'info'              => 'LIVE',
		)
	);

	// Register a new field in the "bankful_field_test_cb" section, inside the "bankful" page.
	add_settings_field(
		'bankful_field_live_validated_account_cc',
		__( 'Live API Connected Account Credit Card:', 'bankful' ),
		'bankful_field_test_cb',
		'bankful',
		'bankful_section_settings',
		array(
			'label_for'         => 'bankful_field_live_validated_account_cc',
			'class'             => 'bankful_row hidden ',
			'type'              => 'text',
			'desc'              => '',
			'default'           => 'no',
		)
	);

	// Register a new field in the "bankful_field_test_cb" section, inside the "bankful" page.
	add_settings_field(
		'bankful_field_live_validated_account_hg',
		__( 'Live API Connected Account Hosted Gateway:', 'bankful' ),
		'bankful_field_test_cb',
		'bankful',
		'bankful_section_settings',
		array(
			'label_for'         => 'bankful_field_live_validated_account_hg',
			'class'             => 'bankful_row hidden ',
			'type'              => 'text',
			'desc'              => '',
			'default'           => 'no',
		)
	);

	// Register a new field in the "bankful_field_test_cb" section, inside the "bankful" page.
	add_settings_field(
		'bankful_field_live_username',
		__( 'LIVE API Username', 'bankful' ),
		'bankful_field_test_cb',
		'bankful',
		'bankful_section_settings',
		array(
			'label_for'         => 'bankful_field_live_username',
			'class'             => 'bankful_row ' . $hidden_validated_live,
			'type' => 'text',
			'desc' => 'Add Live Username here.',
			'default' => '',
		)
	);

	// Register a new field in the "bankful_field_test_cb" section, inside the "bankful" page.
	add_settings_field(
		'bankful_field_live_password',
		__( 'LIVE API Password', 'bankful' ),
		'bankful_field_test_cb',
		'bankful',
		'bankful_section_settings',
		array(
			'label_for'         => 'bankful_field_live_password',
			'class'             => 'bankful_row ' . $hidden_validated_live,
			'type' => 'password',
			'desc' => 'Add Live Password here.',
			'default' => '',
		)
	);

	// Register a new field in the "bankful_field_live_subscription_status" section, inside the "bankful" page.
	add_settings_field(
		'bankful_field_live_subscription_status',
		__( 'Live Subscription Status', 'bankful' ),
		'bankful_field_test_cb',
		'bankful',
		'bankful_section_settings',
		array(
			'label_for'         => 'bankful_field_live_subscription_status',
			'class'             => 'bankful_row hidden ',
			'type'              => 'text',
			'desc'              => '',
			'default'           => 'no',
		)
	);

	// Register a new field in the "bankful_field_live_subscription_status" section, inside the "bankful" page.
	add_settings_field(
		'bankful_field_test_subscription_status',
		__( 'Test Subscription Status', 'bankful' ),
		'bankful_field_test_cb',
		'bankful',
		'bankful_section_settings',
		array(
			'label_for'         => 'bankful_field_test_subscription_status',
			'class'             => 'bankful_row hidden ',
			'type'              => 'text',
			'desc'              => '',
			'default'           => 'no',
		)
	);
};

/**
 * Bankful Select Environment field callback function.
 *
 * @param array $args
 */
function bankful_field_dropdown( $args ) {
	$options = get_option('bankful_options');
	$items = array( 'Live', 'Test' );
	echo "<select id='bankful_field_environment' name='bankful_options[bankful_field_environment]'>";
	foreach ($items as $item) {
		$selected = ( strtolower($options['bankful_field_environment'])==strtolower($item) ) ? 'selected="selected"' : '';
		echo "<option value='" . esc_attr($item) . "' " . esc_attr($selected) . '>' . esc_html($item) . '</option>';
	}
	?>
	</select>
	<?php
}

/**
 * Bankful field callback function.
 *
 * @param array $args
 */
function bankful_field_test_cb( $args ) {
	// Get the value of the setting we've registered with register_setting()
	$options = get_option( 'bankful_options' );
	?>
	<input
		id="<?php echo esc_attr( $args['label_for'] ); ?>"
		name="bankful_options[<?php echo esc_attr( $args['label_for'] ); ?>]"
		value="<?php echo isset( $options[ $args['label_for'] ] ) ? esc_attr($options[ $args['label_for'] ]) : esc_attr($args['default']); ?>"
		type="<?php echo esc_attr( $args['type'] ); ?>"
		class="<?php echo isset( $args['class'] ) ? esc_attr($args['class'] ) : ''; ?> "
		<?php echo ( isset($args['info']) && !empty( $args['info'] ) ) ? 'readonly' : ''; ?>
	/>
	<?php
	if ( isset($args['info']) && !empty( $args['info'] ) && ( !empty( $options[ 'bankful_field_test_validate' ] ) || !empty( $options[ 'bankful_field_live_validate' ] ) )) {

		$accounts_array = array();
		( isset($options['bankful_field_test_validated_account_cc']) && 'yes' == $options['bankful_field_test_validated_account_cc'] ) ?
		array_push($accounts_array, 'Credit Card') : '';
		( isset($options['bankful_field_test_validated_account_hg']) && 'yes' == $options['bankful_field_test_validated_account_hg'] ) ?
		array_push($accounts_array, 'Hosted Page') : '';

		$accounts_array_live = array();
		( isset($options['bankful_field_live_validated_account_cc']) && 'yes' == $options['bankful_field_live_validated_account_cc'] ) ?
		array_push($accounts_array_live, 'Credit Card') : '';
		( isset($options['bankful_field_live_validated_account_hg']) && 'yes' == $options['bankful_field_live_validated_account_hg'] ) ?
		array_push($accounts_array_live, 'Hosted Page') : '';

		echo ( 'TEST' == $args['info'] ) ? ( ( isset($options['bankful_field_test_subscription_status']) && 'yes' == $options['bankful_field_test_subscription_status'] ) ?
		'<span class="dashicons dashicons-update-alt" title="Test Subscription Enabled"></span>' : '' ) : '';
		echo ( 'LIVE' == $args['info'] ) ? ( ( isset($options['bankful_field_live_subscription_status']) && 'yes' == $options['bankful_field_live_subscription_status'] ) ?
		'<span class="dashicons dashicons-update-alt" title="Live Subscription Enabled"></span>' : '' ) : '';
		?>
		<span class="dashicons dashicons-yes-alt" style="color: green;" title="Activated"></span>
		<span class="block"><?php echo esc_html($args['info']); ?> Environment credentials activated with accounts:
		<?php echo esc_html(implode(', ', ( 'TEST' == $args['info'] ) ? $accounts_array : $accounts_array_live)); ?>
			(<a id="<?php echo esc_attr($args['info']); ?>_edit" href="">Update</a>)
		</span>
	<?php
	}
	?>
	<p class="description <?php echo esc_attr( $args['label_for'] ); ?>">
		<?php esc_html_e( $args['desc'], 'bankful' ); ?>
	</p>
	<?php
}

/**
 * Developers section callback function.
 *
 * @param array $args  The settings array, defining title, id, callback.
 */
function bankful_section_settings_callback( $args ) {
	?>
	<p id="<?php echo esc_attr( $args['id'] ); ?>"><?php esc_html_e( 'Here you can define TEST and LIVE Credentials for your plugin', 'bankful' ); ?></p>
	<?php
}

function bankful_activate() {
	global $woocommerce;
	global $wp_version;

	$request_url = 'https://api.paybybankful.com/api/woocommerce/record-plugin-activity';

	$params = array(
		'woo_version' => isset($woocommerce->version) ? $woocommerce->version : '',
		'request_type' => 'A',
		'bf_site_url' => get_site_url(),
		'wp_version' => $wp_version,
		'php_version' => phpversion(),
		'bf_plugin_source' => 'woo_market',
		'site_admin_email' =>  get_bloginfo('admin_email'),
		'hash_salt' => '6a631f095832598284f22b5be8c0cc2e',
	);
	$response = wp_remote_post(
		$request_url,
		array(
			'body' => http_build_query($params),
			'headers'     => array(
				'Content-Type' => 'application/x-www-form-urlencoded',
			),
		)
	);

	$body = wp_remote_retrieve_body($response);
}
register_activation_hook( __FILE__, 'bankful_activate' );

function bankful_deactivate() {
	global $woocommerce;
	global $wp_version;

	$request_url = 'https://api.paybybankful.com/api/woocommerce/record-plugin-activity';

	$params = array(
		'woo_version' => $woocommerce->version,
		'request_type' => 'D',
		'bf_site_url' => get_site_url(),
		'wp_version' => $wp_version,
		'php_version' => phpversion(),
		'bf_plugin_source' => 'woo_market',
		'site_admin_email' =>  get_bloginfo('admin_email'),
		'hash_salt' => '6a631f095832598284f22b5be8c0cc2e',
	);
	$response = wp_remote_post(
		$request_url,
		array(
			'body' => http_build_query($params),
			'headers'     => array(
				'Content-Type' => 'application/x-www-form-urlencoded',
			),
		)
	);

	$body = wp_remote_retrieve_body($response);
}
register_deactivation_hook( __FILE__, 'bankful_deactivate' );

// add_action('woocommerce_order_status_changed', 'empty_cart_on_cancel', 10, 3);
// function empty_cart_on_cancel($order_id, $old_status, $new_status) {
// write_logs_bf('woocommerce_cancel_unpaid_order CALLED');
// write_logs_bf($new_status);
// write_logs_bf($old_status);
// WC()->cart->empty_cart();
// die();
// }

add_action( 'before_woocommerce_init', function () {
	if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
		\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
	}
});
